@@ -146,7 +146,7 @@ struct bluetooth_data {
 
 static int audioservice_send(int sk, const bt_audio_msg_header_t *msg);
 static int audioservice_expect(int sk, bt_audio_msg_header_t *outmsg,
-				int expected_type);
+							int expected_type);
 
 static int bluetooth_start(snd_pcm_ioplug_t *io)
 {
@@ -430,6 +430,8 @@ static int bluetooth_hsp_hw_params(snd_pcm_ioplug_t *io,
 {
 	struct bluetooth_data *data = io->private_data;
 	char buf[BT_SUGGESTED_BUFFER_SIZE];
+	struct bt_open_req *open_req = (void *) buf;
+	struct bt_open_rsp *open_rsp = (void *) buf;
 	struct bt_set_configuration_req *req = (void *) buf;
 	struct bt_set_configuration_rsp *rsp = (void *) buf;
 	int err;
@@ -437,17 +439,34 @@ static int bluetooth_hsp_hw_params(snd_pcm_ioplug_t *io,
 	DBG("Preparing with io->period_size=%lu io->buffer_size=%lu",
 					io->period_size, io->buffer_size);
 
+	memset(req, 0, BT_SUGGESTED_BUFFER_SIZE);
+	open_req->h.type = BT_REQUEST;
+	open_req->h.name = BT_OPEN;
+	open_req->h.length = sizeof(*open_req);
+
+	strncpy(open_req->destination, data->alsa_config.device, 18);
+	open_req->seid = BT_A2DP_SEID_RANGE + 1;
+	open_req->lock = (io->stream == SND_PCM_STREAM_PLAYBACK ?
+			BT_WRITE_LOCK : BT_READ_LOCK);
+
+	err = audioservice_send(data->server.fd, &open_req->h);
+	if (err < 0)
+		return err;
+
+	open_rsp->h.length = sizeof(*open_rsp);
+	err = audioservice_expect(data->server.fd, &open_rsp->h,
+					BT_OPEN);
+	if (err < 0)
+		return err;
+
 	memset(req, 0, BT_SUGGESTED_BUFFER_SIZE);
 	req->h.type = BT_REQUEST;
 	req->h.name = BT_SET_CONFIGURATION;
 	req->h.length = sizeof(*req);
 
-	strncpy(req->device, data->alsa_config.device, 18);
 	req->codec.transport = BT_CAPABILITIES_TRANSPORT_SCO;
+	req->codec.seid = BT_A2DP_SEID_RANGE + 1;
 	req->codec.length = sizeof(pcm_capabilities_t);
-	req->access_mode = (io->stream == SND_PCM_STREAM_PLAYBACK ?
-			BT_CAPABILITIES_ACCESS_MODE_WRITE :
-			BT_CAPABILITIES_ACCESS_MODE_READ);
 
 	req->h.length += req->codec.length - sizeof(req->codec);
 	err = audioservice_send(data->server.fd, &req->h);
@@ -460,7 +479,7 @@ static int bluetooth_hsp_hw_params(snd_pcm_ioplug_t *io,
 	if (err < 0)
 		return err;
 
-	data->transport = rsp->transport;
+	data->transport = BT_CAPABILITIES_TRANSPORT_SCO;
 	data->link_mtu = rsp->link_mtu;
 
 	return 0;
@@ -503,7 +522,7 @@ static uint8_t default_bitpool(uint8_t freq, uint8_t mode)
 }
 
 static int bluetooth_a2dp_init(struct bluetooth_data *data,
-				snd_pcm_hw_params_t *params)
+					snd_pcm_hw_params_t *params)
 {
 	struct bluetooth_alsa_config *cfg = &data->alsa_config;
 	sbc_capabilities_t *cap = &data->a2dp.sbc_capabilities;
@@ -671,6 +690,8 @@ static int bluetooth_a2dp_hw_params(snd_pcm_ioplug_t *io,
 	struct bluetooth_data *data = io->private_data;
 	struct bluetooth_a2dp *a2dp = &data->a2dp;
 	char buf[BT_SUGGESTED_BUFFER_SIZE];
+	struct bt_open_req *open_req = (void *) buf;
+	struct bt_open_rsp *open_rsp = (void *) buf;
 	struct bt_set_configuration_req *req = (void *) buf;
 	struct bt_set_configuration_rsp *rsp = (void *) buf;
 	int err;
@@ -678,6 +699,26 @@ static int bluetooth_a2dp_hw_params(snd_pcm_ioplug_t *io,
 	DBG("Preparing with io->period_size=%lu io->buffer_size=%lu",
 					io->period_size, io->buffer_size);
 
+	memset(req, 0, BT_SUGGESTED_BUFFER_SIZE);
+	open_req->h.type = BT_REQUEST;
+	open_req->h.name = BT_OPEN;
+	open_req->h.length = sizeof(*open_req);
+
+	strncpy(open_req->destination, data->alsa_config.device, 18);
+	open_req->seid = a2dp->sbc_capabilities.capability.seid;
+	open_req->lock = (io->stream == SND_PCM_STREAM_PLAYBACK ?
+			BT_WRITE_LOCK : BT_READ_LOCK);
+
+	err = audioservice_send(data->server.fd, &open_req->h);
+	if (err < 0)
+		return err;
+
+	open_rsp->h.length = sizeof(*open_rsp);
+	err = audioservice_expect(data->server.fd, &open_rsp->h,
+					BT_OPEN);
+	if (err < 0)
+		return err;
+
 	err = bluetooth_a2dp_init(data, params);
 	if (err < 0)
 		return err;
@@ -687,15 +728,11 @@ static int bluetooth_a2dp_hw_params(snd_pcm_ioplug_t *io,
 	req->h.name = BT_SET_CONFIGURATION;
 	req->h.length = sizeof(*req);
 
-	strncpy(req->device, data->alsa_config.device, 18);
 	memcpy(&req->codec, &a2dp->sbc_capabilities,
 			sizeof(a2dp->sbc_capabilities));
 
 	req->codec.transport = BT_CAPABILITIES_TRANSPORT_A2DP;
 	req->codec.length = sizeof(a2dp->sbc_capabilities);
-	req->access_mode = (io->stream == SND_PCM_STREAM_PLAYBACK ?
-			BT_CAPABILITIES_ACCESS_MODE_WRITE :
-			BT_CAPABILITIES_ACCESS_MODE_READ);
 	req->h.length += req->codec.length - sizeof(req->codec);
 
 	err = audioservice_send(data->server.fd, &req->h);
@@ -708,7 +745,7 @@ static int bluetooth_a2dp_hw_params(snd_pcm_ioplug_t *io,
 	if (err < 0)
 		return err;
 
-	data->transport = rsp->transport;
+	data->transport = BT_CAPABILITIES_TRANSPORT_A2DP;
 	data->link_mtu = rsp->link_mtu;
 
 	/* Setup SBC encoder now we agree on parameters */
@@ -921,8 +958,7 @@ done:
 
 static snd_pcm_sframes_t bluetooth_a2dp_read(snd_pcm_ioplug_t *io,
 				const snd_pcm_channel_area_t *areas,
-				snd_pcm_uframes_t offset,
-				snd_pcm_uframes_t size)
+				snd_pcm_uframes_t offset, snd_pcm_uframes_t size)
 {
 	snd_pcm_uframes_t ret = 0;
 	return ret;
@@ -970,7 +1006,8 @@ static snd_pcm_sframes_t bluetooth_a2dp_write(snd_pcm_ioplug_t *io,
 	struct bluetooth_a2dp *a2dp = &data->a2dp;
 	snd_pcm_sframes_t ret = 0;
 	unsigned int bytes_left;
-	int frame_size, encoded, written;
+	int frame_size, encoded;
+	size_t written;
 	uint8_t *buff;
 
 	DBG("areas->step=%u areas->first=%u offset=%lu size=%lu",
@@ -1547,7 +1584,7 @@ static int audioservice_recv(int sk, bt_audio_msg_header_t *inmsg)
 }
 
 static int audioservice_expect(int sk, bt_audio_msg_header_t *rsp,
-				int expected_name)
+							int expected_name)
 {
 	bt_audio_error_t *error;
 	int err = audioservice_recv(sk, rsp);
@@ -1586,7 +1623,8 @@ static int bluetooth_parse_capabilities(struct bluetooth_data *data,
 		return 0;
 
 	while (bytes_left > 0) {
-		if (codec->type == BT_A2DP_CODEC_SBC)
+		if ((codec->type == BT_A2DP_SBC_SINK) &&
+				!(codec->lock & BT_WRITE_LOCK))
 			break;
 
 		bytes_left -= codec->length;
@@ -1602,8 +1640,8 @@ static int bluetooth_parse_capabilities(struct bluetooth_data *data,
 	return 0;
 }
 
-static int bluetooth_init(struct bluetooth_data *data, snd_pcm_stream_t stream,
-				snd_config_t *conf)
+static int bluetooth_init(struct bluetooth_data *data,
+				snd_pcm_stream_t stream, snd_config_t *conf)
 {
 	int sk, err;
 	struct bluetooth_alsa_config *alsa_conf = &data->alsa_config;
@@ -1652,7 +1690,7 @@ static int bluetooth_init(struct bluetooth_data *data, snd_pcm_stream_t stream,
 
 	if (alsa_conf->autoconnect)
 		req->flags |= BT_FLAG_AUTOCONNECT;
-	strncpy(req->device, alsa_conf->device, 18);
+	strncpy(req->destination, alsa_conf->device, 18);
 	if (alsa_conf->has_transport)
 		req->transport = alsa_conf->transport;
 	else
@@ -1731,7 +1769,8 @@ SND_PCM_PLUGIN_DEFINE_FUNC(bluetooth)
 	return 0;
 
 error:
-	bluetooth_exit(data);
+	if (data)
+		bluetooth_exit(data);
 
 	return err;
 }
