@@ -93,13 +93,13 @@ static int response_and_hold = -1;
 
 static struct indicator ofono_indicators[] =
 {
-	{ "battchg",	"0-5",	5 },
-	{ "signal",	"0-5",	5 },
-	{ "service",	"0,1",	1 },
-	{ "call",	"0,1",	0 },
-	{ "callsetup",	"0-3",	0 },
-	{ "callheld",	"0-2",	0 },
-	{ "roam",	"0,1",	0 },
+	{ "battchg",	"0-5",	5,	TRUE },
+	{ "signal",	"0-5",	5,	TRUE },
+	{ "service",	"0,1",	1,	TRUE },
+	{ "call",	"0,1",	0,	TRUE },
+	{ "callsetup",	"0-3",	0,	TRUE },
+	{ "callheld",	"0-2",	0,	FALSE },
+	{ "roam",	"0,1",	0,	TRUE },
 	{ NULL }
 };
 
@@ -888,19 +888,16 @@ static void hal_battery_level_reply(DBusPendingCall *call, void *user_data)
 					" is %d", *value);
 
 	if ((battchg_design > 0 || battchg_last > 0) && battchg_cur >= 0) {
-		int new, cur, max;
+		int new, max;
 
 		if (battchg_last > 0)
 			max = battchg_last;
 		else
 			max = battchg_design;
 
-		cur = telephony_get_indicator(ofono_indicators, "battchg");
 		new = battchg_cur * 5 / max;
 
-		if (new != cur)
-			telephony_update_indicator(ofono_indicators,
-							"battchg", new);
+		telephony_update_indicator(ofono_indicators, "battchg", new);
 	}
 done:
 	dbus_message_unref(reply);
