@@ -53,7 +53,10 @@
 #include "manager.h"
 #include "storage.h"
 
-static int child_pipe[2];
+static int child_pipe[2] = { -1, -1 };
+
+static guint child_io_id = 0;
+static guint ctl_io_id = 0;
 
 static gboolean child_exit(GIOChannel *io, GIOCondition cond, void *user_data)
 {
@@ -100,39 +103,6 @@ static void configure_device(int index)
 		return;
 	}
 
-	/* Set device name */
-	if ((main_opts.flags & (1 << HCID_SET_NAME)) && main_opts.name) {
-		change_local_name_cp cp;
-
-		memset(cp.name, 0, sizeof(cp.name));
-		expand_name((char *) cp.name, sizeof(cp.name),
-						main_opts.name, index);
-
-		hci_send_cmd(dd, OGF_HOST_CTL, OCF_CHANGE_LOCAL_NAME,
-					CHANGE_LOCAL_NAME_CP_SIZE, &cp);
-	}
-
-	/* Set device class */
-	if ((main_opts.flags & (1 << HCID_SET_CLASS))) {
-		write_class_of_dev_cp cp;
-		uint32_t class;
-		uint8_t cls[3];
-
-		if (read_local_class(&di.bdaddr, cls) < 0) {
-			class = htobl(main_opts.class);
-			cls[2] = get_service_classes(&di.bdaddr);
-			memcpy(cp.dev_class, &class, 3);
-		} else {
-			if (!(main_opts.scan & SCAN_INQUIRY))
-				cls[1] &= 0xdf; /* Clear discoverable bit */
-			cls[2] = get_service_classes(&di.bdaddr);
-			memcpy(cp.dev_class, cls, 3);
-		}
-
-		hci_send_cmd(dd, OGF_HOST_CTL, OCF_WRITE_CLASS_OF_DEV,
-					WRITE_CLASS_OF_DEV_CP_SIZE, &cp);
-	}
-
 	/* Set page timeout */
 	if ((main_opts.flags & (1 << HCID_SET_PAGETO))) {
 		write_page_timeout_cp cp;
@@ -277,22 +247,24 @@ static int init_known_adapters(int ctl)
 {
 	struct hci_dev_list_req *dl;
 	struct hci_dev_req *dr;
-	int i;
+	int i, err;
 
 	dl = g_try_malloc0(HCI_MAX_DEV * sizeof(struct hci_dev_req) + sizeof(uint16_t));
 	if (!dl) {
-		info("Can't allocate devlist buffer: %s (%d)",
+		err = -errno;
+		error("Can't allocate devlist buffer: %s (%d)",
 							strerror(errno), errno);
-		return errno;
+		return err;
 	}
 
 	dl->dev_num = HCI_MAX_DEV;
 	dr = dl->dev_req;
 
 	if (ioctl(ctl, HCIGETDEVLIST, (void *) dl) < 0) {
-		info("Can't get device list: %s (%d)",
+		err = -errno;
+		error("Can't get device list: %s (%d)",
 							strerror(errno), errno);
-		return errno;
+		return err;
 	}
 
 	for (i = 0; i < dl->dev_num; i++, dr++) {
@@ -359,26 +331,31 @@ static int hciops_setup(void)
 	struct sockaddr_hci addr;
 	struct hci_filter flt;
 	GIOChannel *ctl_io, *child_io;
-	int sock;
+	int sock, err;
+
+	if (child_pipe[0] != -1)
+		return -EALREADY;
 
 	if (pipe(child_pipe) < 0) {
+		err = -errno;
 		error("pipe(): %s (%d)", strerror(errno), errno);
-		return errno;
+		return err;
 	}
 
 	child_io = g_io_channel_unix_new(child_pipe[0]);
 	g_io_channel_set_close_on_unref(child_io, TRUE);
-	g_io_add_watch(child_io,
-			G_IO_IN | G_IO_ERR | G_IO_HUP | G_IO_NVAL,
-			child_exit, NULL);
+	child_io_id = g_io_add_watch(child_io,
+				G_IO_IN | G_IO_ERR | G_IO_HUP | G_IO_NVAL,
+				child_exit, NULL);
 	g_io_channel_unref(child_io);
 
 	/* Create and bind HCI socket */
 	sock = socket(AF_BLUETOOTH, SOCK_RAW, BTPROTO_HCI);
 	if (sock < 0) {
+		err = -errno;
 		error("Can't open HCI socket: %s (%d)", strerror(errno),
 								errno);
-		return errno;
+		return err;
 	}
 
 	/* Set filter */
@@ -387,8 +364,9 @@ static int hciops_setup(void)
 	hci_filter_set_event(EVT_STACK_INTERNAL, &flt);
 	if (setsockopt(sock, SOL_HCI, HCI_FILTER, &flt,
 							sizeof(flt)) < 0) {
+		err = -errno;
 		error("Can't set filter: %s (%d)", strerror(errno), errno);
-		return errno;
+		return err;
 	}
 
 	memset(&addr, 0, sizeof(addr));
@@ -396,15 +374,16 @@ static int hciops_setup(void)
 	addr.hci_dev = HCI_DEV_NONE;
 	if (bind(sock, (struct sockaddr *) &addr,
 							sizeof(addr)) < 0) {
+		err = -errno;
 		error("Can't bind HCI socket: %s (%d)",
 							strerror(errno), errno);
-		return errno;
+		return err;
 	}
 
 	ctl_io = g_io_channel_unix_new(sock);
 	g_io_channel_set_close_on_unref(ctl_io, TRUE);
 
-	g_io_add_watch(ctl_io, G_IO_IN, io_stack_event, NULL);
+	ctl_io_id = g_io_add_watch(ctl_io, G_IO_IN, io_stack_event, NULL);
 
 	g_io_channel_unref(ctl_io);
 
@@ -414,6 +393,25 @@ static int hciops_setup(void)
 
 static void hciops_cleanup(void)
 {
+	if (child_io_id) {
+		g_source_remove(child_io_id);
+		child_io_id = 0;
+	}
+
+	if (ctl_io_id) {
+		g_source_remove(ctl_io_id);
+		ctl_io_id = 0;
+	}
+
+	if (child_pipe[0] >= 0) {
+		close(child_pipe[0]);
+		child_pipe[0] = -1;
+	}
+
+	if (child_pipe[1] >= 0) {
+		close(child_pipe[1]);
+		child_pipe[1] = -1;
+	}
 }
 
 static int hciops_start(int index)
@@ -516,13 +514,36 @@ static int hciops_discoverable(int index)
 	return 0;
 }
 
-static int hciops_set_limited_discoverable(int index, const uint8_t *cls,
+static int hciops_set_class(int index, uint32_t class)
+{
+	int dd, err;
+	write_class_of_dev_cp cp;
+
+	dd = hci_open_dev(index);
+	if (dd < 0)
+		return -EIO;
+
+	memcpy(cp.dev_class, &class, 3);
+
+	err = hci_send_cmd(dd, OGF_HOST_CTL, OCF_WRITE_CLASS_OF_DEV,
+					WRITE_CLASS_OF_DEV_CP_SIZE, &cp);
+
+	if (err < 0)
+		err = -errno;
+
+	hci_close_dev(dd);
+
+	return err;
+}
+
+static int hciops_set_limited_discoverable(int index, uint32_t class,
 							gboolean limited)
 {
 	int dd, err = 0;
-	uint32_t dev_class;
 	int num = (limited ? 2 : 1);
 	uint8_t lap[] = { 0x33, 0x8b, 0x9e, 0x00, 0x8b, 0x9e };
+	write_current_iac_lap_cp cp;
+
 	/*
 	 * 1: giac
 	 * 2: giac + liac
@@ -531,34 +552,16 @@ static int hciops_set_limited_discoverable(int index, const uint8_t *cls,
 	if (dd < 0)
 		return -EIO;
 
-	if (hci_write_current_iac_lap(dd, num, lap, HCI_REQ_TIMEOUT) < 0) {
-		err = -errno;
-		error("Can't write current IAC LAP: %s(%d)",
-						strerror(errno), errno);
-		goto done;
-	}
+	memset(&cp, 0, sizeof(cp));
+	cp.num_current_iac = num;
+	memcpy(&cp.lap, lap, num * 3);
 
-	if (limited) {
-		if (cls[1] & 0x20)
-			goto done; /* Already limited */
+	err = hci_send_cmd(dd, OGF_HOST_CTL, OCF_WRITE_CURRENT_IAC_LAP,
+					(num * 3 + 1), &cp);
 
-		dev_class = (cls[2] << 16) | ((cls[1] | 0x20) << 8) | cls[0];
-	} else {
-		if (!(cls[1] & 0x20))
-			goto done; /* Already clear */
-
-		dev_class = (cls[2] << 16) | ((cls[1] & 0xdf) << 8) | cls[0];
-	}
-
-	if (hci_write_class_of_dev(dd, dev_class, HCI_REQ_TIMEOUT) < 0) {
-		err = -errno;
-		error("Can't write class of device: %s (%d)",
-						strerror(errno), errno);
-		goto done;
-	}
-done:
 	hci_close_dev(dd);
-	return err;
+
+	return hciops_set_class(index, class);
 }
 
 static int hciops_start_discovery(int index, gboolean periodic)
@@ -628,6 +631,90 @@ static int hciops_stop_discovery(int index)
 	return err;
 }
 
+static int hciops_resolve_name(int index, bdaddr_t *bdaddr)
+{
+	remote_name_req_cp cp;
+	int dd, err = 0;
+
+	dd = hci_open_dev(index);
+	if (dd < 0)
+		return -EIO;
+
+	memset(&cp, 0, sizeof(cp));
+	bacpy(&cp.bdaddr, bdaddr);
+	cp.pscan_rep_mode = 0x02;
+
+	err = hci_send_cmd(dd, OGF_LINK_CTL, OCF_REMOTE_NAME_REQ,
+					REMOTE_NAME_REQ_CP_SIZE, &cp);
+	if (err < 0)
+		err = -errno;
+
+	hci_close_dev(dd);
+
+	return err;
+}
+
+static int hciops_set_name(int index, const char *name)
+{
+	change_local_name_cp cp;
+	int dd, err = 0;
+
+	dd = hci_open_dev(index);
+	if (dd < 0)
+		return -EIO;
+
+	memset(&cp, 0, sizeof(cp));
+	strncpy((char *) cp.name, name, sizeof(cp.name));
+
+	err = hci_send_cmd(dd, OGF_HOST_CTL, OCF_CHANGE_LOCAL_NAME,
+					CHANGE_LOCAL_NAME_CP_SIZE, &cp);
+	if (err < 0)
+		err = -errno;
+
+	hci_close_dev(dd);
+
+	return err;
+}
+
+static int hciops_read_name(int index)
+{
+	int dd, err = 0;
+
+	dd = hci_open_dev(index);
+	if (dd < 0)
+		return -EIO;
+
+	err = hci_send_cmd(dd, OGF_HOST_CTL, OCF_READ_LOCAL_NAME, 0, 0);
+	if (err < 0)
+		err = -errno;
+
+	hci_close_dev(dd);
+
+	return err;
+}
+
+static int hciops_cancel_resolve_name(int index, bdaddr_t *bdaddr)
+{
+	remote_name_req_cancel_cp cp;
+	int dd, err = 0;
+
+	dd = hci_open_dev(index);
+	if (dd < 0)
+		return -EIO;
+
+	memset(&cp, 0, sizeof(cp));
+	bacpy(&cp.bdaddr, bdaddr);
+
+	err = hci_send_cmd(dd, OGF_LINK_CTL, OCF_REMOTE_NAME_REQ_CANCEL,
+					REMOTE_NAME_REQ_CANCEL_CP_SIZE, &cp);
+	if (err < 0)
+		err = -errno;
+
+	hci_close_dev(dd);
+
+	return err;
+}
+
 static struct btd_adapter_ops hci_ops = {
 	.setup = hciops_setup,
 	.cleanup = hciops_cleanup,
@@ -639,6 +726,11 @@ static struct btd_adapter_ops hci_ops = {
 	.set_limited_discoverable = hciops_set_limited_discoverable,
 	.start_discovery = hciops_start_discovery,
 	.stop_discovery = hciops_stop_discovery,
+	.resolve_name = hciops_resolve_name,
+	.cancel_resolve_name = hciops_cancel_resolve_name,
+	.set_name = hciops_set_name,
+	.read_name = hciops_read_name,
+	.set_class = hciops_set_class,
 };
 
 static int hciops_init(void)
