@@ -63,10 +63,12 @@ static gboolean add_plugin(void *handle, struct bluetooth_plugin_desc *desc)
 		return FALSE;
 
 	if (g_str_equal(desc->version, VERSION) == FALSE) {
-		DBG("version mismatch for %s", desc->name);
+		error("Version mismatch for %s", desc->name);
 		return FALSE;
 	}
 
+	debug("Loading %s plugin", desc->name);
+
 	plugin = g_try_new0(struct bluetooth_plugin, 1);
 	if (plugin == NULL)
 		return FALSE;
@@ -88,6 +90,9 @@ static gboolean is_disabled(const char *name, char **list)
 		char *str;
 		gboolean equal;
 
+		if (g_str_equal(name, list[i]))
+			return TRUE;
+
 		str = g_strdup_printf("%s.so", list[i]);
 
 		equal = g_str_equal(str, name);
@@ -101,12 +106,15 @@ static gboolean is_disabled(const char *name, char **list)
 	return FALSE;
 }
 
+#include "builtin.h"
+
 gboolean plugin_init(GKeyFile *config)
 {
 	GSList *list;
 	GDir *dir;
 	const gchar *file;
 	gchar **disabled;
+	unsigned int i;
 
 	if (strlen(PLUGINDIR) == 0)
 		return FALSE;
@@ -122,6 +130,16 @@ gboolean plugin_init(GKeyFile *config)
 	else
 		disabled = NULL;
 
+	debug("Loading builtin plugins");
+
+	for (i = 0; __bluetooth_builtin[i]; i++) {
+		if (disabled && is_disabled(__bluetooth_builtin[i]->name,
+								disabled))
+			continue;
+
+		add_plugin(NULL,  __bluetooth_builtin[i]);
+	}
+
 	debug("Loading plugins %s", PLUGINDIR);
 
 	dir = g_dir_open(PLUGINDIR, 0, NULL);
@@ -134,7 +152,6 @@ gboolean plugin_init(GKeyFile *config)
 		struct bluetooth_plugin_desc *desc;
 		void *handle;
 		gchar *filename;
-		struct stat st;
 
 		if (g_str_has_prefix(file, "lib") == TRUE ||
 				g_str_has_suffix(file, ".so") == FALSE)
@@ -145,13 +162,6 @@ gboolean plugin_init(GKeyFile *config)
 
 		filename = g_build_filename(PLUGINDIR, file, NULL);
 
-		if (stat(filename, &st) < 0) {
-			error("Can't find plugin %s: %s", filename,
-							strerror(errno));
-			g_free(filename);
-			continue;
-		}
-
 		handle = dlopen(filename, RTLD_NOW);
 		if (handle == NULL) {
 			error("Can't load plugin %s: %s", filename,
@@ -201,7 +211,8 @@ void plugin_cleanup(void)
 		if (plugin->active == TRUE && plugin->desc->exit)
 			plugin->desc->exit();
 
-		dlclose(plugin->handle);
+		if (plugin->handle != NULL)
+			dlclose(plugin->handle);
 
 		g_free(plugin);
 	}
